#include <winsock2.h>
#include <windows.h>

#include "game.hpp"
#include "sound.hpp"
#include "network.hpp"

CGame gGame;

void CGame::Init()
{
  
  //pick first piece
  //clear the gameboard
  for( int i=0; i<game_w; i++)
    for( int j=0; j<game_h; j++ )
      gameboard[i][j]=0;

  //put in some virii
  virus_count=18;
  int x,y;
  for( int i=0; i<virus_count; i++ )
  {
    do
    {
    x=rand()%game_w;
    y=rand()%6+9;
    }while( gameboard[x][y]!=0 );
    gameboard[x][y]=rand()%3+redvirus;
  }

  nextpiece[0]=rand()%3;
  nextpiece[1]=rand()%3;

  gameboard[game_w/2][0]=rand()%3+red;
  gameboard[game_w/2][1]=rand()%3+red;

  current_x=game_w/2;
  current_y=1;

  orientation=Vert;

  fallingblocks=false;

  lost=false;
  /*
  //TEST just playing
  fallingblocks=true;
  */
}

void CGame::CheckForLines()
{
  //invalidate the current position
  current_x=-1;

  int current_score=0;

  int currentrun;
  int currentcolor;
  int current_check;

  //check vertical lines
  for( int i=0; i<game_w; i++ )
  for( int j=game_h-1; j>2; j-- )
  {
    currentrun=0;
    switch( gGame.gameboard[i][j]&0xFF )
    {
    case redvirus:
    case red:
      currentcolor=0;
      break;
    case bluevirus:
    case blue:
      currentcolor=1;
      break;
    case yellowvirus:
    case yellow:
      currentcolor=2;
      break;
    default:
      //not a playable block move on
      currentrun=-1;
      break;
    }
    current_check=j-1;
    while( currentrun<3 && currentrun!=-1 )
    {
      switch( gGame.gameboard[i][current_check]&0xFF )
      {
      case redvirus:
      case red:
        currentcolor==0 ? currentrun++ : currentrun=-1;
        break;
      case bluevirus:
      case blue:
        currentcolor==1 ? currentrun++ : currentrun=-1;
        break;
      case yellowvirus:
      case yellow:
        currentcolor==2 ? currentrun++ : currentrun=-1;
        break;
      default:
        currentrun=-1;
        break;
      }
      current_check--;
    }

    if( currentrun != -1 )
    {
      bool stillmatch=true;
      //we have a line
      current_check=j;
      do{
        //mark for death and give some score
        gGame.gameboard[i][current_check]|=(0x4<<8);
        current_score++;

        //detach any blocks from this block
        switch( (gGame.gameboard[i][current_check]>>8)&0x3 )
        {
        case 1:
          //detach block on right
          gGame.gameboard[i+1][current_check]&=(0xFFFF^(0x3<<8));
          break;
        case 2:
          //deatch block on left
          gGame.gameboard[i-1][current_check]&=(0xFFFF^(0x3<<8));
          break;
        default:
          //nothing to do
          break;
        }

        current_check--;
        switch( gGame.gameboard[i][current_check]&0xF )
        {
        case redvirus:
        case red:
          stillmatch=(currentcolor==0);
          break;
        case bluevirus:
        case blue:
          stillmatch=(currentcolor==1);
          break;
        case yellowvirus:
        case yellow:
          stillmatch=(currentcolor==2);
          break;
        default:
          stillmatch=false;
          break;
        }        
      }while(stillmatch);
    }
  }

  //check horizontals
  for( int i=0; i<game_w-3; i++ )
  for( int j=0; j<game_h; j++ )
  {
    currentrun=0;
    switch( gGame.gameboard[i][j]&0xFF )
    {
    case redvirus:
    case red:
      currentcolor=0;
      break;
    case bluevirus:
    case blue:
      currentcolor=1;
      break;
    case yellowvirus:
    case yellow:
      currentcolor=2;
      break;
    default:
      //not a playable block move on
      currentrun=-1;
      break;
    }
    current_check=i+1;
    while( currentrun<3 && currentrun!=-1 )
    {
      switch( gGame.gameboard[current_check][j]&0xFF )
      {
      case redvirus:
      case red:
        currentcolor==0 ? currentrun++ : currentrun=-1;
        break;
      case bluevirus:
      case blue:
        currentcolor==1 ? currentrun++ : currentrun=-1;
        break;
      case yellowvirus:
      case yellow:
        currentcolor==2 ? currentrun++ : currentrun=-1;
        break;
      default:
        currentrun=-1;
        break;
      }
      current_check++;
    }


    if( currentrun != -1 )
    {
      bool stillmatch=true;
      //we have a line
      current_check=i;
      do{
        //mark for death and give some score
        gGame.gameboard[current_check][j]|=(0x4<<8);
        current_score++;

        //detach any blocks from this block
        switch( (gGame.gameboard[current_check][j]>>8)&0x3 )
        {
        case 1:
          //detach block on right
          gGame.gameboard[current_check+1][j]&=(0xFFFF^(0x3<<8));
          break;
        case 2:
          //deatch block on left
          gGame.gameboard[current_check-1][j]&=(0xFFFF^(0x3<<8));
          break;
        default:
          //nothing to do
          break;
        }

        current_check++;
        switch( gGame.gameboard[current_check][j]&0xF )
        {
        case redvirus:
        case red:
          stillmatch=(currentcolor==0);
          break;
        case bluevirus:
        case blue:
          stillmatch=(currentcolor==1);
          break;
        case yellowvirus:
        case yellow:
          stillmatch=(currentcolor==2);
          break;
        default:
          stillmatch=false;
          break;
        }        
      }while(stillmatch);
    }

  }

  //remove all blocks marked for death
  for( int i=0; i<game_w; i++ )
  for( int j=0; j<game_h; j++ )
  {
    if( gGame.gameboard[i][j]&(4<<8) )
    {
      if( (gGame.gameboard[i][j]&0xFF) >= redvirus && (gGame.gameboard[i][j]&0xFF) <= yellowvirus )
      {
        gGame.virus_count--;
      }

      gGame.gameboard[i][j]=0;
    }
  }

  score+=current_score;
  if( current_score == 0 )
  {
    if( attacks.empty() )
    {
      gGame.GetNextPiece();
    }

    //send out attacks
    Packet packet;
    packet.type=pckAttack;
    if( score > 11 )
    {
      packet.data.attack.amount=3;
      gSound.PlaySample( CSoundSystem::attackstrong );
      gNetwork.SendPacket( &packet );
    }else if( score > 7 )
    {
      //mini attack
      packet.data.attack.amount=2;
      gSound.PlaySample( CSoundSystem::attack );
      gNetwork.SendPacket( &packet );
    }
    score=0;
  }else{
    //you scored
    gSound.PlaySample( CSoundSystem::clearblocks );
    gGame.fallingblocks=true;
  }

}

void CGame::GetNextPiece()
{
  if( gameboard[game_w/2][0] != 0 )
  {
    lost=true;
    current_x=-1;
  }

  //check for any blocks to remove
  gameboard[game_w/2][0]=nextpiece[0]+red;
  gameboard[game_w/2][1]=nextpiece[1]+red;
  
  current_x=game_w/2;
  current_y=1;
  
  nextpiece[0]=rand()%3;
  nextpiece[1]=rand()%3;

  orientation=Vert;
}