#include <winsock2.h>
#include <windows.h>
#include <stdio.h>

#include "resource.h"
#include "gfx.hpp"
#include "error.h"
#include "input.h"
#include "network.hpp"
#include "sound.hpp"
#include "globals.hpp"
#include "game.hpp"

// Window class and caption text
char g_szClass[]   = "Relax";
char g_szCaption[] = "Doctor Feelgood";

// Function prototypes
int PASCAL WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR szCmdLine, int nCmdShow);
long FAR PASCAL WindowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL AppInit(HWND hWnd,HINSTANCE hInst);
void AppShutdown();
void MainLoop(HWND hWnd);


enum eImages{ imgHost=0, imgJoin, imgQuit, 
              imgNoGames, imgFoundGame, imgCursor,
              imgWaiting, imgBackground, imgBlueVirus,
              imgBlueVirus2, imgBlueVirus3, imgRedVirus,
              imgRedVirus2, imgRedVirus3, imgYellowVirus,
              imgYellowVirus2, imgYellowVirus3, imgWin,
              imgLose, imgIntro, imgHostJoin,
              imgRedBlock, imgBlueBlock, imgYellowBlock,
              imgGeneric, imgQtoQuit, imgSingle };
const int numimages=27;
int virusanim=0;
LPDIRECT3DTEXTURE8 *images;
void LoadImages();
void UnloadImages();

HRESULT Render();
HRESULT FrameMove();

void DrawCombat();
void MoveCombat();
void UserMoveBlock();
void UserBlockGravity();
void LetBlocksFall();

GameState gGameState;


void LoadImages()
{
  images=new LPDIRECT3DTEXTURE8[numimages];
  for( int i=0; i<numimages; i++ )
  {
    images[i]=NULL;
  }

  HRESULT hr;
  hr=D3DXCreateTextureFromFile( g_pD3DDevice, "media\\host.jpg", &images[imgHost] );
  hr=D3DXCreateTextureFromFile( g_pD3DDevice, "media\\foundgame.jpg", &images[imgFoundGame] );
  hr=D3DXCreateTextureFromFile( g_pD3DDevice, "media\\join.jpg", &images[imgJoin] );
  hr=D3DXCreateTextureFromFile( g_pD3DDevice, "media\\nogames.jpg", &images[imgNoGames] );
  hr=D3DXCreateTextureFromFileEx( g_pD3DDevice, "media\\cursor.bmp", 0, 0,
                                  1, 0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED,
                                  D3DX_DEFAULT, D3DX_DEFAULT, D3DCOLOR_XRGB(255,255,255),
                                  NULL, NULL, &images[imgCursor] );
  hr=D3DXCreateTextureFromFile( g_pD3DDevice, "media\\waiting.jpg", &images[imgWaiting] );
  hr=D3DXCreateTextureFromFile( g_pD3DDevice, "media\\background1.bmp", &images[imgBackground] );

  hr=D3DXCreateTextureFromFileEx( g_pD3DDevice, "media\\redvirus1.bmp", 0, 0,
                                  1, 0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED,
                                  D3DX_DEFAULT, D3DX_DEFAULT, D3DCOLOR_XRGB(255,255,255),
                                  NULL, NULL, &images[imgRedVirus] );
  hr=D3DXCreateTextureFromFileEx( g_pD3DDevice, "media\\redvirus2.bmp", 0, 0,
                                  1, 0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED,
                                  D3DX_DEFAULT, D3DX_DEFAULT, D3DCOLOR_XRGB(255,255,255),
                                  NULL, NULL, &images[imgRedVirus2] );
  hr=D3DXCreateTextureFromFileEx( g_pD3DDevice, "media\\redvirus3.bmp", 0, 0,
                                  1, 0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED,
                                  D3DX_DEFAULT, D3DX_DEFAULT, D3DCOLOR_XRGB(255,255,255),
                                  NULL, NULL, &images[imgRedVirus3] );

  hr=D3DXCreateTextureFromFileEx( g_pD3DDevice, "media\\bluevirus1.bmp", 0, 0,
                                  1, 0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED,
                                  D3DX_DEFAULT, D3DX_DEFAULT, D3DCOLOR_XRGB(255,255,255),
                                  NULL, NULL, &images[imgBlueVirus] );
  hr=D3DXCreateTextureFromFileEx( g_pD3DDevice, "media\\bluevirus2.bmp", 0, 0,
                                  1, 0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED,
                                  D3DX_DEFAULT, D3DX_DEFAULT, D3DCOLOR_XRGB(255,255,255),
                                  NULL, NULL, &images[imgBlueVirus2] );
  hr=D3DXCreateTextureFromFileEx( g_pD3DDevice, "media\\bluevirus3.bmp", 0, 0,
                                  1, 0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED,
                                  D3DX_DEFAULT, D3DX_DEFAULT, D3DCOLOR_XRGB(255,255,255),
                                  NULL, NULL, &images[imgBlueVirus3] );

  hr=D3DXCreateTextureFromFileEx( g_pD3DDevice, "media\\yellowvirus1.bmp", 0, 0,
                                  1, 0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED,
                                  D3DX_DEFAULT, D3DX_DEFAULT, D3DCOLOR_XRGB(255,255,255),
                                  NULL, NULL, &images[imgYellowVirus] );
  hr=D3DXCreateTextureFromFileEx( g_pD3DDevice, "media\\yellowvirus1.bmp", 0, 0,
                                  1, 0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED,
                                  D3DX_DEFAULT, D3DX_DEFAULT, D3DCOLOR_XRGB(255,255,255),
                                  NULL, NULL, &images[imgYellowVirus2] );
  hr=D3DXCreateTextureFromFileEx( g_pD3DDevice, "media\\yellowvirus3.bmp", 0, 0,
                                  1, 0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED,
                                  D3DX_DEFAULT, D3DX_DEFAULT, D3DCOLOR_XRGB(255,255,255),
                                  NULL, NULL, &images[imgYellowVirus3] );

  hr=D3DXCreateTextureFromFile( g_pD3DDevice, "media\\red.bmp", &images[imgRedBlock] );
  hr=D3DXCreateTextureFromFile( g_pD3DDevice, "media\\blue.bmp", &images[imgBlueBlock] );
  hr=D3DXCreateTextureFromFile( g_pD3DDevice, "media\\yellow.bmp", &images[imgYellowBlock] );\
  
  hr=D3DXCreateTextureFromFile( g_pD3DDevice, "media\\win.bmp", &images[imgWin] );
  hr=D3DXCreateTextureFromFile( g_pD3DDevice, "media\\lose.bmp", &images[imgLose] );

  hr=D3DXCreateTextureFromFile( g_pD3DDevice, "media\\intro.jpg", &images[imgIntro] );
  hr=D3DXCreateTextureFromFile( g_pD3DDevice, "media\\hostjoin.jpg", &images[imgHostJoin] );
  hr=D3DXCreateTextureFromFile( g_pD3DDevice, "media\\generic.bmp", &images[imgGeneric] );
  
  hr=D3DXCreateTextureFromFile( g_pD3DDevice, "media\\quit.bmp", &images[imgQtoQuit] );
  hr=D3DXCreateTextureFromFile( g_pD3DDevice, "media\\single.bmp", &images[imgSingle] );
}

void UnloadImages()
{
  for( int i=0; i<numimages; i++ )
  {
    if( images[i] )
      images[i]->Release();
  }

  delete [] images;
}

int PASCAL WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR szCmdLine, int nCmdShow)
{
  WNDCLASSEX wcex;
  HWND       hWnd;


  // Initialize the COM system
  CoInitialize(NULL);

  // Create the window class here and register it
  wcex.cbSize        = sizeof(wcex);
  wcex.style         = CS_CLASSDC;
  wcex.lpfnWndProc   = WindowProc;
  wcex.cbClsExtra    = 0;
  wcex.cbWndExtra    = 0;
  wcex.hInstance     = hInst;
  wcex.hIcon         = LoadIcon(NULL, MAKEINTRESOURCE(IDI_APP));
  wcex.hCursor       = LoadCursor(NULL, IDC_ARROW);
  wcex.hbrBackground = NULL;
  wcex.lpszMenuName  = NULL;
  wcex.lpszClassName = g_szClass;
  wcex.hIconSm       = LoadIcon(NULL, MAKEINTRESOURCE(IDI_APP));
  if(!RegisterClassEx(&wcex))
    return FALSE;

  hWnd = CreateWindow(g_szClass, g_szCaption,
    WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX,
    0, 0, 640, 480,
    NULL, NULL, hInst, NULL);
  if(!hWnd)
    return FALSE;
  ShowWindow(hWnd, SW_NORMAL);
  UpdateWindow(hWnd);


  // Call init function and enter message pump
  if(AppInit(hWnd,hInst))
  {
    MainLoop(hWnd);
  }

  // Call shutdown
  AppShutdown();

  // Unregister the window class
  UnregisterClass(g_szClass, hInst);

  // Shut down the COM system
  CoUninitialize();

  return 0;
}

long FAR PASCAL WindowProc(HWND hWnd, UINT uMsg,              \
                           WPARAM wParam, LPARAM lParam)
{
  // Only handle window destruction messages
  switch(uMsg) {
    case WM_DESTROY:
      PostQuitMessage(0);
      break;
      
    case WM_SETCURSOR:
      g_Mouse.HandleSetCursor();
      break;

    default:
      return (long)DefWindowProc(hWnd, uMsg, wParam, lParam);
  }

  return 0;
}

BOOL AppInit(HWND hWnd,HINSTANCE hInst)
{
  // Initialize Direct3D (instance,device,handle,force window, multithreaded?)
  //also sets the projection matrix
#ifdef _DEBUG
  if(FAILED(InitD3D(&g_pD3D, &g_pD3DDevice, hWnd, TRUE, TRUE)))
#else
    if(FAILED(InitD3D(&g_pD3D, &g_pD3DDevice, hWnd, FALSE, TRUE)))
#endif
  {
    SetError("Device could not be created");
    return FALSE;
  }

  InitializeInput(g_pD3DDevice,hWnd,hInst);

  D3DXMATRIX matWorld;
  D3DXMatrixIdentity(&matWorld);
  g_pD3DDevice->SetTransform(D3DTS_WORLD,&matWorld);

  g_pD3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE,  TRUE);
  g_pD3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
  g_pD3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
  g_pD3DDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_MODULATE);
  g_pD3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE,TRUE);

  CameraType=FPS;

  //let the real cursor show
  g_Mouse.ShowCursor( false );

  g_Mouse.SetCursorPosition( 100, 100 );

  gSound.Init(hWnd);
  gSound.PlayMusic();

  gNetwork.Init();
  gGameState=Intro;
  //gGameState=HostOrJoin;
  //gGameState=Combat;
  LoadImages();

  gGame.Init();


  return TRUE;
}

void AppShutdown()
{
  gNetwork.Stop();
  gSound.Shutdown();
  UnloadImages();
  ShutdownInput();
  ShutdownD3D();
}

void MainLoop(HWND hWnd)
{
  MSG        Msg;

  // Start message pump, waiting for user to exit
  ZeroMemory(&Msg, sizeof(MSG));

  while(Msg.message != WM_QUIT)
  {
    if(PeekMessage(&Msg, NULL, 0, 0, PM_REMOVE))
    {
      TranslateMessage(&Msg);
      DispatchMessage(&Msg);
    }

    // Do some work here
    BASS_Update();
    FrameMove();
    Render();
  }
}

HRESULT Render()
{

  //make view matrix here
  D3DXMATRIX view;
  switch(CameraType)
  {
  case FPS:
    break;
  default:
    break;
    //D3DXMatrixLookAtLH(&view,INSERT OTHER PARAMS);
  }

  g_pD3DDevice->Clear(0,0,D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER,\
    D3DCOLOR_XRGB(0,0,0),1.0f,0);

  //turn off lighting
  g_pD3DDevice->SetRenderState(D3DRS_LIGHTING,FALSE);

  g_pD3DDevice->BeginScene();

  float top;
  float left;
  int x, y;

  switch( gGameState )
  {
  case Intro:
    top=0;
    left=0;
    BlitRect( images[imgIntro], left, top, left+800, top+600, D3DCOLOR_XRGB(255,255,255), 0.0f );
    
    left=20;
    top=200;
    BlitRect( images[imgBlueVirus+virusanim], left, top, left+128, top+128, D3DCOLOR_ARGB(255,255,255,255), 0.0f );
    
    left=800-20-128;
    top=200;
    BlitRect( images[imgRedVirus+virusanim], left, top, left+128, top+128, D3DCOLOR_ARGB(255,255,255,255), 0.0f );
    
    break;
  case HostOrJoin: 
    top=0;
    left=0;
    BlitRect( images[imgHostJoin], left, top, left+800, top+600, D3DCOLOR_XRGB(255,255,255), 0.0f );
    top=150;
    left=800/2-520/2;
    BlitRect( images[imgSingle], left, top, left+520, top+150, D3DCOLOR_XRGB(255,255,255), 0.0f );
    top=325;
    left=800/2-170/2;
    BlitRect( images[imgHost], left, top, left+170, top+100, D3DCOLOR_XRGB(255,255,255), 0.0f );
    top=450;
    BlitRect( images[imgJoin], left, top, left+170, top+100, D3DCOLOR_XRGB(255,255,255), 0.0f );
    break;
  case WaitForClient:
    BlitRect( images[imgGeneric], 0, 0, 800, 600, D3DCOLOR_XRGB(255,255,255), 0.0f );
    top=150;
    left=800/2-600/2;
    BlitRect( images[imgWaiting], left, top, left+600, top+160, D3DCOLOR_XRGB(255,255,255), 0.0f );
    break;
  case NoGames:
    BlitRect( images[imgGeneric], 0, 0, 800, 600, D3DCOLOR_XRGB(255,255,255), 0.0f );
    BlitRect( images[imgQtoQuit], 50, 350, 50+700, 350+100, D3DCOLOR_XRGB(255,255,255), 0.0f ); 
    top=150;
    left=800/2-580/2;
    BlitRect( images[imgNoGames], left, top, left+580, top+100, D3DCOLOR_XRGB(255,255,255), 0.0f );
    break;
  case YouWin:
    BlitRect( images[imgGeneric], 0, 0, 800, 600, D3DCOLOR_XRGB(255,255,255), 0.0f );
    BlitRect( images[imgQtoQuit], 50, 350, 50+700, 350+100, D3DCOLOR_XRGB(255,255,255), 0.0f ); 
    top=150;
    left=800/2-520/2;
    BlitRect( images[imgWin], left, top, left+580, top+170, D3DCOLOR_XRGB(255,255,255), 0.0f );
    break;
  case YouLose:
    BlitRect( images[imgGeneric], 0, 0, 800, 600, D3DCOLOR_XRGB(255,255,255), 0.0f );
    BlitRect( images[imgQtoQuit], 50, 400, 50+700, 400+100, D3DCOLOR_XRGB(255,255,255), 0.0f ); 
    top=150;
    left=800/2-520/2;
    BlitRect( images[imgLose], left, top, left+520, top+200, D3DCOLOR_XRGB(255,255,255), 0.0f );
    break;
  case Combat:
    DrawCombat();
    break;
  default:
    SetError("game state not handled");
  }

  g_Mouse.GetCursorPosition( x, y );
  top=(float)y;
  left=(float)x;
  BlitRect( images[imgCursor], left, top, left+16, top+32, D3DCOLOR_ARGB(255,255,255,255), 0.0f );

  g_pD3DDevice->EndScene();

  return g_pD3DDevice->Present(NULL,NULL,NULL,NULL);

}

HRESULT FrameMove()
{
  //animate the viruses
  static DWORD animtime=timeGetTime();
  if( timeGetTime()-animtime > 100 )
  {
    virusanim++;
    virusanim%=3;
    animtime=timeGetTime();
  }

  g_Mouse.Poll();
  g_Mouse.UpdateCursorPos();

  /*
  POINT pt=g_Mouse.GetMousePos();
*/
  POINT pt;
  int x=0,y=0;
  g_Mouse.GetCursorPosition( x, y );
  pt.x=x;
  pt.y=y;

  static DWORD introtime;

  switch( gGameState )
  {
  case Intro:
    if( g_Keyboard.IsAnyKeyDown() )
    {
      gGameState=HostOrJoin;
      introtime=timeGetTime();
    }
    break;
  case HostOrJoin:
    if( timeGetTime() - introtime < 100 )
    {
      break;
    }
    if( g_Keyboard.IsKeyDown( DIK_S ) )
    {
      gGameState=Combat;
    }else if( g_Keyboard.IsKeyDown(DIK_H) )
    {
      gNetwork.ListenForGame();
      gGameState=WaitForClient;
    }else if( g_Keyboard.IsKeyDown(DIK_J) )
    {
      if( gNetwork.BrowseForGame() )
      {
        gGameState=Combat;
      }else{
        gGameState=NoGames;
      }
    }else if( g_Mouse.IsButtonDown(0) )
    {
      //host
      if( pt.x > 800/2-170/2 && pt.x < 800/2+170/2 )
        if( pt.y > 325 && pt.y < 425 )
        {
          gNetwork.ListenForGame();
          gGameState=WaitForClient;
        }
      //join
      if( pt.x > 800/2-170/2 && pt.x < 800/2+170/2 )
        if( pt.y > 450 && pt.y < 550 )
        {
          if( gNetwork.BrowseForGame() )
          {
            gGameState=Combat;
          }else{
            gGameState=NoGames;
          }
        }
      //single player
      if( pt.x > 800/2-520/2 && pt.x < 800/2+520/2 )
        if( pt.y > 150 && pt.y < 300 )
        {
          gGameState=Combat;
        }
    }
    break;
  case WaitForClient:
    if( g_Keyboard.IsKeyDown( DIK_ESCAPE ) )
    {
      PostQuitMessage(0);
    }
    if( gNetwork.FoundGame() )
    {
      gGameState=Combat;
    }
    break;
  case YouWin:
  case YouLose:
  case NoGames:
    if( g_Keyboard.IsKeyDown( DIK_Q ) )
    {
      PostQuitMessage(0);
    }
    break;
  case Combat:
    MoveCombat();
    break;
  default:
    SetError("game state not handled");
  }

  return S_OK;
}

void MoveCombat()
{
  static DWORD cheattime=timeGetTime();
  if( g_Keyboard.IsKeyDown( DIK_P ) && timeGetTime()-cheattime>100 )
  {
    Packet packet;
    packet.type=pckAttack;
    packet.data.attack.amount=2;
    gNetwork.SendPacket( &packet );
    cheattime=timeGetTime();
  }
  if( g_Keyboard.IsKeyDown( DIK_O ) && timeGetTime()-cheattime>100 )
  {
    Packet packet;
    packet.type=pckAttack;
    packet.data.attack.amount=3;
    gNetwork.SendPacket( &packet );
    cheattime=timeGetTime();
  }

  //animate the viruses
  static DWORD animtime=timeGetTime();
  if( timeGetTime()-animtime > 150 )
  {
    virusanim++;
    virusanim%=3;
    animtime=timeGetTime();
  }

  Packet packet;

  //see if you won
  if( gGame.virus_count==0 )
  {
    //tell the other player that you owned him
    packet.type=pckGameOver;
    packet.data.WinOrLose=true;
    gNetwork.SendPacket( &packet );
    gGameState=YouWin;
    gSound.PlaySample( CSoundSystem::win );
  }
  //or if you lost
  if( gGame.lost )
  {
    packet.type=pckGameOver;
    packet.data.WinOrLose=false;
    gNetwork.SendPacket( &packet );
    gGameState=YouLose;
    gSound.PlaySample( CSoundSystem::lose );
  }

  //handle network events
  gNetwork.ReceivePackets();
  while( !gNetwork.packets.empty() )
  {
    packet=gNetwork.packets.front();
    gNetwork.packets.pop();

    switch( packet.type )
    {
    case pckGameOver:
      if( packet.data.WinOrLose )
      {
        //other player won
        PrintError("you lost\n");
        gSound.PlaySample( CSoundSystem::lose );
        gGameState=YouLose;
      }else{
        //other player lost
        PrintError("you win\n");
        gGameState=YouWin;
      }
      break;
    case pckAttack:
      gGame.attacks.push( packet.data.attack.amount );
      break;
    default:
#ifdef _DEBUG
      SetError( "got a packet that was unrecognized" );
#endif
      break;
    }
  }

  //apply any attacks
  if( !gGame.fallingblocks && !gGame.attacks.empty() && gGame.current_x==-1 )
  {
    switch( gGame.attacks.front() )
    {
    case 2:
      gGame.gameboard[2][0]=timeGetTime()%3+red;
      gGame.gameboard[5][0]=timeGetTime()%3+red;
      break;
    case 3:
      gGame.gameboard[2][0]=timeGetTime()%3+red;
      gGame.gameboard[4][0]=timeGetTime()%3+red;
      gGame.gameboard[6][0]=timeGetTime()%3+red;
      break;
    default:
      SetError("Bad Attack");
    }
    gGame.attacks.pop();
    gGame.fallingblocks=true;
  }

  //follow through with falling
  if( gGame.fallingblocks )
  {
    LetBlocksFall();
  }

  if( !gGame.fallingblocks )
  {
    //let user move block
    UserMoveBlock();

    //drop users block
    UserBlockGravity();
  }

}

void DrawCombat()
{
  float top, left;

  top=0;
  left=0;
  //draw the bg
  BlitRect( images[imgBackground], left, top, left+800, top+600, D3DCOLOR_XRGB(255,255,255), 0.0f );

  //draw the game board
  for( int i=0; i<game_w; i++)
    for( int j=0; j<game_h; j++ )
    {
      top=j*32+60;
      left=i*32+270;
      //use the low byte
      switch( gGame.gameboard[i][j] & 0xFF )
      {
      case redvirus:
        BlitRect( images[imgRedVirus+virusanim], left, top, left+32, top+32, D3DCOLOR_ARGB(255,255,255,255), 0.0f );
        break;
      case bluevirus:
        BlitRect( images[imgBlueVirus+virusanim], left, top, left+32, top+32, D3DCOLOR_ARGB(255,255,255,255), 0.0f );
        break;
      case yellowvirus:
        BlitRect( images[imgYellowVirus+virusanim], left, top, left+32, top+32, D3DCOLOR_ARGB(255,255,255,255), 0.0f );
        break;
      case red:
        BlitRect( images[imgRedBlock], left, top, left+32, top+32, D3DCOLOR_ARGB(255,255,255,255), 0.0f );
        switch((gGame.gameboard[i][j]>>8)&3)
        {
        case 1:
          BlitRect( NULL, left+26, top+6, left+32, top+26, D3DCOLOR_ARGB(255,255,0,0), 0.0f );
          break;
        case 2:
          BlitRect( NULL, left, top+6, left+4, top+26, D3DCOLOR_ARGB(255,255,0,0), 0.0f );
          break;
        default:
          break;
        }
        break;
      case blue:
        BlitRect( images[imgBlueBlock], left, top, left+32, top+32, D3DCOLOR_ARGB(255,255,255,255), 0.0f );
        switch((gGame.gameboard[i][j]>>8)&3)
        {
        case 1:
          BlitRect( NULL, left+26, top+6, left+32, top+26, D3DCOLOR_ARGB(255,0,0,255), 0.0f );
          break;
        case 2:
          BlitRect( NULL, left, top+6, left+4, top+26, D3DCOLOR_ARGB(255,0,0,255), 0.0f );
          break;
        default:
          break;
        }
        break;
      case yellow:
        BlitRect( images[imgYellowBlock], left, top, left+32, top+32, D3DCOLOR_ARGB(255,255,255,255), 0.0f );
        switch((gGame.gameboard[i][j]>>8)&3)
        {
        case 1:
          BlitRect( NULL, left+26, top+6, left+32, top+26, D3DCOLOR_ARGB(255,255,255,0), 0.0f );
          break;
        case 2:
          BlitRect( NULL, left, top+6, left+4, top+26, D3DCOLOR_ARGB(255,255,255,0), 0.0f );
          break;
        default:
          break;
        }
        break;
      default:
        //don't draw anything
        break;
      }
    }

    //draw the pill connector
    if( gGame.current_x != -1 )
    {
      switch( gGame.orientation )
      {
      case Vert:
        //top half
        top=(gGame.current_y-1)*32+60;
        left=gGame.current_x*32+270;
        switch((gGame.gameboard[gGame.current_x][gGame.current_y-1]&0xFF))
        {
        case blue:
          BlitRect( NULL, left+6, top+26, left+26, top+32, D3DCOLOR_XRGB(0,0,255), 0.0f );
          break;
        case yellow:
          BlitRect( NULL, left+6, top+26, left+26, top+32, D3DCOLOR_XRGB(255,255,0), 0.0f );
          break;
        case red:
          BlitRect( NULL, left+6, top+26, left+26, top+32, D3DCOLOR_XRGB(255,0,0), 0.0f );
          break;
        }
        //bottom half
        top=(gGame.current_y)*32+60;
        left=gGame.current_x*32+270;
        switch((gGame.gameboard[gGame.current_x][gGame.current_y]&0xFF))
        {
        case blue:
          BlitRect( NULL, left+6, top, left+26, top+6, D3DCOLOR_XRGB(0,0,255), 0.0f );
          break;
        case yellow:
          BlitRect( NULL, left+6, top, left+26, top+6, D3DCOLOR_XRGB(255,255,0), 0.0f );
          break;
        case red:
          BlitRect( NULL, left+6, top, left+26, top+6, D3DCOLOR_XRGB(255,0,0), 0.0f );
          break;
        }
        break;
      case Hor:
        //left half
        top=gGame.current_y*32+60;
        left=gGame.current_x*32+270;
        switch((gGame.gameboard[gGame.current_x][gGame.current_y]&0xFF))
        {
        case blue:
          BlitRect( NULL, left+26, top+6, left+32, top+26, D3DCOLOR_XRGB(0,0,255), 0.0f );
          break;
        case yellow:
          BlitRect( NULL, left+26, top+6, left+32, top+26, D3DCOLOR_XRGB(255,255,0), 0.0f );
          break;
        case red:
          BlitRect( NULL, left+26, top+6, left+32, top+26, D3DCOLOR_XRGB(255,0,0), 0.0f );
          break;
        }
        //right half
        top=gGame.current_y*32+60;
        left=(gGame.current_x+1)*32+270;
        switch((gGame.gameboard[gGame.current_x+1][gGame.current_y]&0xFF))
        {
        case blue:
          BlitRect( NULL, left, top+6, left+6, top+26, D3DCOLOR_XRGB(0,0,255), 0.0f );
          break;
        case yellow:
          BlitRect( NULL, left, top+6, left+6, top+26, D3DCOLOR_XRGB(255,255,0), 0.0f );
          break;
        case red:
          BlitRect( NULL, left, top+6, left+6, top+26, D3DCOLOR_XRGB(255,0,0), 0.0f );
          break;
        }
        break;
      }
    }

    //draw the next piece
    left=50+48;
    top=130+32;
    BlitRect( images[imgRedBlock+gGame.nextpiece[0]], left, top, left+32, top+32, D3DCOLOR_ARGB(255,255,255,255), 0.0f );
    switch( gGame.nextpiece[0] )
    {
    case 0:
      BlitRect( NULL, left+6, top+26, left+26, top+32, D3DCOLOR_XRGB(255,0,0), 0.0f );
      break;
    case 1:
      BlitRect( NULL, left+6, top+26, left+26, top+32, D3DCOLOR_XRGB(0,0,255), 0.0f );
      break;
    case 2:
      BlitRect( NULL, left+6, top+26, left+26, top+32, D3DCOLOR_XRGB(255,255,0), 0.0f );
      break;
    default:
      SetError("bad next piece");
      break;
    }
    top+=32;
    BlitRect( images[imgRedBlock+gGame.nextpiece[1]], left, top, left+32, top+32, D3DCOLOR_ARGB(255,255,255,255), 0.0f );
    switch( gGame.nextpiece[1] )
    {
    case 0:
      BlitRect( NULL, left+6, top, left+26, top+6, D3DCOLOR_XRGB(255,0,0), 0.0f );
      break;
    case 1:
      BlitRect( NULL, left+6, top, left+26, top+6, D3DCOLOR_XRGB(0,0,255), 0.0f );
      break;
    case 2:
      BlitRect( NULL, left+6, top, left+26, top+6, D3DCOLOR_XRGB(255,255,0), 0.0f );
      break;
    default:
      SetError("bad next piece");
      break;
    }

    //draw the virii
    left=50;
    top=360;
    BlitRect( images[imgBlueVirus+virusanim], left, top, left+128, top+128, D3DCOLOR_ARGB(255,255,255,255), 0.0f );
    left=750-128;
    top=120;
    BlitRect( images[imgRedVirus+virusanim], left, top, left+128, top+128, D3DCOLOR_ARGB(255,255,255,255), 0.0f );
    left=750-128;
    top=360;
    BlitRect( images[imgYellowVirus+virusanim], left, top, left+128, top+128, D3DCOLOR_ARGB(255,255,255,255), 0.0f );

}

void UserMoveBlock()
{
  static bool actionlast[4]={false,false,false,false};
  bool allowaction[4];
  for(int i=0;i<4;i++)
  {
    allowaction[i]=!actionlast[i];
    actionlast[i]=false;
  }

  //let user change orientation
  if( g_Keyboard.IsKeyDown( DIK_F ) )
  {
    if( !allowaction[0] )
    {
      //hold the key block
      actionlast[0]=true;
    }else
    //counter clockwise
    if( gGame.orientation == Vert )
    {
      if( gGame.current_x<game_w-1 )
      {
        //make sure its open
        if( gGame.gameboard[gGame.current_x+1][gGame.current_y]==0 )
        {
          gGame.orientation=Hor;
          gGame.gameboard[gGame.current_x+1][gGame.current_y]=gGame.gameboard[gGame.current_x][gGame.current_y];
          gGame.gameboard[gGame.current_x][gGame.current_y]=gGame.gameboard[gGame.current_x][gGame.current_y-1];
          gGame.gameboard[gGame.current_x][gGame.current_y-1]=0;
          actionlast[0]=true;
        }
      }else if( gGame.gameboard[gGame.current_x-1][gGame.current_y]==0 )
      {
        gGame.orientation=Hor;
        gGame.gameboard[gGame.current_x-1][gGame.current_y]=gGame.gameboard[gGame.current_x][gGame.current_y-1];
        gGame.gameboard[gGame.current_x][gGame.current_y-1]=0;
        gGame.current_x--;
        actionlast[0]=true;
      }
    }else{
      if( gGame.gameboard[gGame.current_x][gGame.current_y-1]==0 )
      {
        gGame.orientation=Vert;
        gGame.gameboard[gGame.current_x][gGame.current_y-1]=gGame.gameboard[gGame.current_x+1][gGame.current_y];
        gGame.gameboard[gGame.current_x+1][gGame.current_y]=0;
        actionlast[0]=true;
      }
    }
  }

  if( g_Keyboard.IsKeyDown( DIK_G ) )
  {
    if( !allowaction[1] )
    {
      //hold the key block
      actionlast[1]=true;
    }else
    //clockwise
    if( gGame.orientation == Vert )
    {
      if( gGame.current_x<game_w-1 )
      {
        //make sure its open
        if( gGame.gameboard[gGame.current_x+1][gGame.current_y]==0 )
        {
          gGame.orientation=Hor;
          gGame.gameboard[gGame.current_x+1][gGame.current_y]=gGame.gameboard[gGame.current_x][gGame.current_y-1];
          gGame.gameboard[gGame.current_x][gGame.current_y-1]=0;
          actionlast[1]=true;
        }
      }else if( gGame.gameboard[gGame.current_x-1][gGame.current_y]==0 )
      {
        gGame.orientation=Hor;
        gGame.gameboard[gGame.current_x-1][gGame.current_y]=gGame.gameboard[gGame.current_x][gGame.current_y];
        gGame.gameboard[gGame.current_x][gGame.current_y]=gGame.gameboard[gGame.current_x][gGame.current_y-1];
        gGame.gameboard[gGame.current_x][gGame.current_y-1]=0;
        gGame.current_x--;
        actionlast[1]=true;
      }
    }else{
      if( gGame.gameboard[gGame.current_x][gGame.current_y-1]==0 )
      {
        gGame.orientation=Vert;
        gGame.gameboard[gGame.current_x][gGame.current_y-1]=gGame.gameboard[gGame.current_x][gGame.current_y];
        gGame.gameboard[gGame.current_x][gGame.current_y]=gGame.gameboard[gGame.current_x+1][gGame.current_y];
        gGame.gameboard[gGame.current_x+1][gGame.current_y]=0;
        actionlast[1]=true;
      }
    }
  }

  if( g_Keyboard.IsKeyDown( DIK_LEFT ) )
  {
    if( !allowaction[2] )
    {
      //hold the key block
      actionlast[2]=true;
    }else
    //verify that can move left
    switch( gGame.orientation )
    {
    case Vert:
      if( gGame.current_x>0 && gGame.gameboard[gGame.current_x-1][gGame.current_y]==0 &&
        gGame.gameboard[gGame.current_x-1][gGame.current_y-1]==0 )
      {
        gGame.gameboard[gGame.current_x-1][gGame.current_y]=gGame.gameboard[gGame.current_x][gGame.current_y];
        gGame.gameboard[gGame.current_x-1][gGame.current_y-1]=gGame.gameboard[gGame.current_x][gGame.current_y-1];
        gGame.gameboard[gGame.current_x][gGame.current_y]=0;
        gGame.gameboard[gGame.current_x][gGame.current_y-1]=0;
        gGame.current_x--;
        actionlast[2]=true;
      }
      break;
    case Hor:
      if( gGame.current_x>0 && gGame.gameboard[gGame.current_x-1][gGame.current_y]==0 )
      {
        gGame.gameboard[gGame.current_x-1][gGame.current_y]=gGame.gameboard[gGame.current_x][gGame.current_y];
        gGame.gameboard[gGame.current_x][gGame.current_y]=gGame.gameboard[gGame.current_x+1][gGame.current_y];
        gGame.gameboard[gGame.current_x+1][gGame.current_y]=0;
        gGame.current_x--;
          actionlast[2]=true;
      }
      break;
    default:
      SetError( "bad case statement" );
    }
  }

  if( g_Keyboard.IsKeyDown( DIK_RIGHT ) )
  {
    if( !allowaction[3] )
    {
      //hold the key block
      actionlast[3]=true;
    }else
    //verify that can move left
    switch( gGame.orientation )
    {
    case Vert:
      if( gGame.current_x<game_w-1 && gGame.gameboard[gGame.current_x+1][gGame.current_y]==0 &&
        gGame.gameboard[gGame.current_x+1][gGame.current_y-1]==0 )
      {
        gGame.gameboard[gGame.current_x+1][gGame.current_y]=gGame.gameboard[gGame.current_x][gGame.current_y];
        gGame.gameboard[gGame.current_x+1][gGame.current_y-1]=gGame.gameboard[gGame.current_x][gGame.current_y-1];
        gGame.gameboard[gGame.current_x][gGame.current_y]=0;
        gGame.gameboard[gGame.current_x][gGame.current_y-1]=0;
        gGame.current_x++;
        actionlast[3]=true;
      }
      break;
    case Hor:
      if( gGame.current_x+1<game_w-1 && gGame.gameboard[gGame.current_x+2][gGame.current_y]==0 )
      {
        gGame.gameboard[gGame.current_x+2][gGame.current_y]=gGame.gameboard[gGame.current_x+1][gGame.current_y];
        gGame.gameboard[gGame.current_x+1][gGame.current_y]=gGame.gameboard[gGame.current_x][gGame.current_y];
        gGame.gameboard[gGame.current_x][gGame.current_y]=0;
        gGame.current_x++;
        actionlast[3]=true;
      }
      break;
    default:
      SetError( "bad case statement" );
    }
  }
}

void UserBlockGravity()
{
  //no user block
  if( gGame.current_x == -1 ) {return;}

  static DWORD droptime=timeGetTime();
  
  if( g_Keyboard.IsKeyDown( DIK_DOWN ) )
  {
    droptime-=100;
  }

  if( timeGetTime()-droptime < 750 )
  {
    return;
  }
  droptime=timeGetTime();
  //drop players block
  switch( gGame.orientation )
  {
  case Vert:
    if( gGame.current_y < game_h-1 && gGame.gameboard[gGame.current_x][gGame.current_y+1]==0 )
    {
      gGame.gameboard[gGame.current_x][gGame.current_y+1]=gGame.gameboard[gGame.current_x][gGame.current_y];
      gGame.gameboard[gGame.current_x][gGame.current_y]=gGame.gameboard[gGame.current_x][gGame.current_y-1];
      gGame.gameboard[gGame.current_x][gGame.current_y-1]=0;
      gGame.current_y++;
    }else{
      //block stopped
      //need to mark each block as unattached
      gGame.gameboard[gGame.current_x][gGame.current_y]&=0xFF;
      gGame.gameboard[gGame.current_x][gGame.current_y-1]&=0xFF;
      gGame.CheckForLines();
    }
    break;
  case Hor:
    if( gGame.current_y < game_h-1 && gGame.gameboard[gGame.current_x][gGame.current_y+1]==0
      && gGame.gameboard[gGame.current_x+1][gGame.current_y+1]==0 )
    {
      gGame.gameboard[gGame.current_x][gGame.current_y+1]=gGame.gameboard[gGame.current_x][gGame.current_y];
      gGame.gameboard[gGame.current_x+1][gGame.current_y+1]=gGame.gameboard[gGame.current_x+1][gGame.current_y];
      
      gGame.gameboard[gGame.current_x][gGame.current_y]=0;
      gGame.gameboard[gGame.current_x+1][gGame.current_y]=0;
      gGame.current_y++;
    }else{
      //need to mark each block as attached
      gGame.gameboard[gGame.current_x][gGame.current_y]|=(1<<8);
      gGame.gameboard[gGame.current_x+1][gGame.current_y]|=(2<<8);
      gGame.CheckForLines();
    }
    break;
  default:
    SetError( "bad case statement" );
  }
}

void LetBlocksFall()
{
  static DWORD droptime=timeGetTime();
  if( timeGetTime()-droptime < 300 )
  {
    return;
  }
  droptime=timeGetTime();

  gGame.fallingblocks=false;

  for( int j=game_h-2; j>=0; j-- )
  for( int i=0; i<game_w; i++ )
  {
    //need to verify that block is unattached
    //this happens automagically because the bit flag increases the value :)
    if( (gGame.gameboard[i][j]&0xFF)>=red && (gGame.gameboard[i][j]&0xFF)<=yellow )
    {
      if( ((gGame.gameboard[i][j]>>8)&3) ==0 && gGame.gameboard[i][j+1] == 0 )
      {
        gGame.gameboard[i][j+1]=gGame.gameboard[i][j];
        gGame.gameboard[i][j]=0;
        //hit the flag again, there may be more blocks to fall
        gGame.fallingblocks=true;
      }else if( ((gGame.gameboard[i][j]>>8)&1) == 1 )
      {
        if( gGame.gameboard[i][j+1]==0 && gGame.gameboard[i+1][j+1] == 0 )
        {
          gGame.gameboard[i][j+1]=gGame.gameboard[i][j];
          gGame.gameboard[i][j]=0;
          gGame.gameboard[i+1][j+1]=gGame.gameboard[i+1][j];
          gGame.gameboard[i+1][j]=0;
          //hit the flag again, there may be more blocks to fall
          gGame.fallingblocks=true;          
        }
      }
    }
  }

  if( !gGame.fallingblocks )
  {
    gGame.CheckForLines();
  }
}