#include "network.hpp"
#include "error.h"

#include <assert.h>
#include <stdio.h>

NetworkSystem gNetwork;

char gamesearch[]="ChillOut";


DWORD WINAPI ListenForGame(void* sock)
{
  SOCKET s=(SOCKET)sock;

  char buf[32];
  memset(buf,0,32);
  int size=sizeof(sockaddr_in);
  do
  {
    if( recvfrom( s, buf, 32, 0, (sockaddr*)&gNetwork.opponent, &size  ) != SOCKET_ERROR )
    {
      if( !strncmp(buf, gamesearch, 32) )
      {
        gNetwork.foundgame=true;
        char msg[256];
        sprintf( msg, "joined game at %s\n", inet_ntoa(gNetwork.opponent.sin_addr) );
        PrintError(msg);
      }
    }
    Sleep(3);
  }while(!gNetwork.foundgame);

  int seed=timeGetTime();
  srand(seed);

  char msg[30];
  sprintf( msg, "%s%i", gamesearch, seed );
  int len=(int)strlen(msg);
  int ret=sendto( s, msg, len , 0, (sockaddr*)&gNetwork.opponent, sizeof(sockaddr_in) );
  if( ret == SOCKET_ERROR )
  {
    SetError( "sendto failed" );
  }

  return 0;
}

void NetworkSystem::ListenForGame()
{
  assert( s != NULL );

  sockaddr_in addr;
  addr.sin_family=AF_INET;
  addr.sin_port=htons(4000);
  addr.sin_addr.s_addr=INADDR_ANY;

  int ret=bind( s, (sockaddr*)&addr, sizeof(sockaddr_in) );
  if( ret==SOCKET_ERROR )
  {
    SetError( "binding error" );
  }

  DWORD id;
  CreateThread( NULL, 0, ::ListenForGame, (void*)s, 0, &id );
}

bool NetworkSystem::BrowseForGame()
{
  assert( s != NULL );

  sockaddr_in addr;
  addr.sin_family=AF_INET;
  addr.sin_port=htons(4000);
  addr.sin_addr.s_addr=INADDR_BROADCAST;

  int len=(int)strlen(gamesearch);
  int ret=sendto( s, gamesearch, len , 0, (sockaddr*)&addr, sizeof(addr) );
  if( ret == SOCKET_ERROR )
  {
    SetError("couldn't sendto");
    ret=WSAGetLastError();
  }

  //wait for recognition
  char buf[32];
  memset(buf,0,32);
  int size=sizeof(sockaddr_in);
  DWORD time=timeGetTime();
  int seed;
  while( timeGetTime()-time<1000 )
  {
    if( recvfrom( s, buf, 32, 0, (sockaddr*)&opponent, &size  ) != SOCKET_ERROR )
    {
      if( !strncmp(buf, gamesearch, len) )
      {
        //extract the random seed
        seed=atoi( &buf[len] );
        srand( seed );

        char msg[256];
        sprintf( msg, "joined game at %s with seed %i\n", inet_ntoa(opponent.sin_addr), seed );
        PrintError(msg);
        return true;
      }
    }
  }

  return false;
}

void NetworkSystem::Init()
{
  // Start Winsock up

  WSAData wsaData;
  if( WSAStartup(MAKEWORD(2, 2), &wsaData) != 0 )
  {
    SetError( "couldn't start winsock" );
  }

  s=socket( AF_INET, SOCK_DGRAM, IPPROTO_UDP );

  if( !s )
    SetError( "couldn't make socket" );
  
  BOOL broadcast=true;
  setsockopt( s, SOL_SOCKET, SO_BROADCAST, (char*)&broadcast, sizeof(BOOL) );
  
  //don't block
  u_long one=1;
  ioctlsocket( s, FIONBIO , &one );

}

void NetworkSystem::Stop()
{
  closesocket(s);
  WSACleanup();
}

void NetworkSystem::SendPacket( Packet* packet )
{
  assert( s != NULL );
  assert( packet != NULL );
  sendto( s, (char*)packet, sizeof(Packet), 0, (sockaddr*)&opponent, sizeof(sockaddr_in) );
}

void NetworkSystem::ReceivePackets()
{
  Packet packet;
  while( recv( s, (char*)&packet, sizeof(Packet), 0 ) != SOCKET_ERROR )
  {
    packets.push( packet );
  }
}